"""

"""

from notecoin.huobi.history.utils import *
from notetool.log import log

ALL_PERIODS = ['1min', '5min', '15min', '30min', '60min', '4hour', '1day']
pre_url = "https://futures.huobi.com/data"
logger = log()

_SWAP = "swap"
_SPOT = "spot"
_FUTURE = "future"
_OPTION = "option"
_LINEARSWAP = "linear-swap"


class HistoryDownload:
    def __init__(self, _pre_url=None,
                 data_type: str = None,
                 all_period: list = None,
                 _type: str = 'spot',
                 freq: str = None,
                 start_date=None,
                 end_date=None):
        self.freq = freq or "daily"
        self.type = _type or 'spot'
        self.data_type = data_type or 'klines'
        self.pre_url = _pre_url or "https://futures.huobi.com/data"
        self.all_period = all_period or ALL_PERIODS
        self.start_date = start_date or datetime(2017, 10, 27)
        self.end_date = end_date or datetime(2021, 7, 27)
        self.all_symbols = None

    def init(self):
        if self.all_symbols is None:
            ok, all_symbols = False, []
            if self.type == _SPOT:
                ok, all_symbols = get_all_spot_symbols()
            elif self.type == _FUTURE:
                ok, all_symbols = get_all_future_symbols()
            elif self.type == _SWAP:
                ok, all_symbols = get_all_swap_symbols()
            elif self.type == _OPTION:
                ok, all_symbols = get_all_option_symbols()
            elif self.type == _LINEARSWAP:
                ok, all_symbols = get_all_linearswap_symbols()

            if not ok:
                logger.warning(all_symbols)
                return
            self.all_symbols = all_symbols

    def _download_daily(self, path_url, symbol, period) -> tuple:
        self.init()
        all_oks = []
        all_errs = []
        interval = self.end_date - self.start_date
        for index in range(interval.days):
            current = self.start_date + timedelta(days=index)
            url = f'{path_url}/{symbol.upper()}-{period}-{current.year}-{current.month:02}-{current.day:02}'
            zip_file = f'{url}.zip'
            # print(zip_file)
            check_file = f'{url}.CHECKSUM'
            ok, msg = http_download(zip_file)
            if not ok:
                all_errs.append({'url': url, 'msg': msg})
            else:
                all_oks.append(zip_file)
            ok, msg = http_download(check_file)
            if not ok:
                all_errs.append({'url': url, 'msg': msg})
            else:
                all_oks.append(url)
        return all_oks, all_errs

    def b_download(self, all_symbols: list = None):
        """return date is: [start, end)"""
        for symbol in all_symbols:
            for period in self.all_period:
                if period in ['trades', ]:
                    path_url = f'{self.pre_url}/{self.data_type}/{self.type}/{self.freq}/{symbol}'
                else:
                    path_url = f'{self.pre_url}/{self.data_type}/{self.type}/{self.freq}/{symbol}/{period}'
                all_oks, all_errs = self._download_daily(path_url, symbol, period)
                logger.warning(f'success:{all_oks}')
                logger.warning(f'failed:{all_errs}')
        logger.info('done')

    def b_download_daily_spot(self, all_symbols: list = None):
        """return date is: [start, end)"""
        self.type = _SPOT
        self.b_download(all_symbols)

    def b_download_daily_future(self, all_symbols: list = None):
        """return date is: [start, end)"""
        self.type = _FUTURE
        self.b_download(all_symbols)

    def b_download_daily_swap(self, all_symbols: list = None):
        """return date is: [start, end)"""
        self.type = _SWAP
        self.b_download(all_symbols)

    def b_download_daily_linearswap(self, all_symbols: list = None):
        """return date is: [start, end)"""
        self.type = _LINEARSWAP
        self.b_download(all_symbols)

    def b_download_daily_option(self, all_symbols: list = None):
        """return date is: [start, end)"""
        self.type = _OPTION
        self.b_download(all_symbols)


if __name__ == "__main__":
    history = HistoryDownload(data_type="klines",
                              start_date=datetime(2021, 5, 21),
                              end_date=datetime(2021, 5, 23),
                              all_period=['1min', '15min']
                              )
    history.b_download()
    # download_daily_future()
    # download_daily_swap()
    # download_daily_linearswap()
    # download_daily_option(all_period=['1min'])



"""

"""
from notecoin.huobi.history.utils import *
from notetool.log import log

pre_url = "https://futures.huobi.com/data"
logger = log()


def b_download(data_type: str,
               all_period: list = None,
               start: datetime = None,
               end: datetime = None,
               all_symbols: list = None,
               spot="spot",
               freq="daily"):
    """return date is: [start, end)"""

    global pre_url
    for symbol in all_symbols:
        for period in all_period:
            if period in ['trades', ]:
                path_url = f'{pre_url}/{data_type}/{spot}/{freq}/{symbol}'
            else:
                path_url = f'{pre_url}/{data_type}/{spot}/{freq}/{symbol}/{period}'
            all_oks, all_errs = download_daily(path_url, symbol, period, start, end)
            logger.warning(f'success:{all_oks}')
            logger.warning(f'failed:{all_errs}')
    logger.info('done')


def b_download_daily_spot(data_type: str,
                          all_period: list = None,
                          start: datetime = None,
                          end: datetime = None,
                          all_symbols: list = None):
    """return date is: [start, end)"""

    global pre_url
    if all_period is None:
        all_period = ALL_PERIODS
    if start is None:
        start = SPOT_START_DATE
    if end is None:
        end = END_DATE

    if all_symbols is None:
        ok, all_symbols = get_all_spot_symbols()
        if not ok:
            logger.warning(all_symbols)
            return

    b_download(data_type, all_period, start, end, all_symbols, spot='spot', freq='daily')


def b_download_daily_future(data_type: str,
                            all_period: list = None,
                            start: datetime = None,
                            end: datetime = None,
                            all_symbols: list = None):
    """return date is: [start, end)"""

    global pre_url
    if all_period is None:
        all_period = ALL_PERIODS
    if start is None:
        start = SPOT_START_DATE
    if end is None:
        end = END_DATE

    if all_symbols is None:
        ok, all_symbols = get_all_future_symbols()
        if not ok:
            logger.warning(all_symbols)
            return

    b_download(data_type, all_period, start, end, all_symbols, spot='future', freq='daily')


def b_download_daily_swap(data_type: str,
                          all_period: list = None,
                          start: datetime = None,
                          end: datetime = None,
                          all_symbols: list = None):
    """return date is: [start, end)"""

    global pre_url
    if all_period is None:
        all_period = ALL_PERIODS
    if start is None:
        start = SPOT_START_DATE
    if end is None:
        end = END_DATE

    if all_symbols is None:
        ok, all_symbols = get_all_swap_symbols()
        if not ok:
            logger.warning(all_symbols)
            return

    b_download(data_type, all_period, start, end, all_symbols, spot='swap', freq='daily')


def b_download_daily_linearswap(data_type: str,
                                all_period: list = None,
                                start: datetime = None,
                                end: datetime = None,
                                all_symbols: list = None):
    """return date is: [start, end)"""

    global pre_url
    if all_period is None:
        all_period = ALL_PERIODS
    if start is None:
        start = SPOT_START_DATE
    if end is None:
        end = END_DATE

    if all_symbols is None:
        ok, all_symbols = get_all_linearswap_symbols()
        if not ok:
            logger.warning(all_symbols)
            return

    b_download(data_type, all_period, start, end, all_symbols, spot='linear-swap', freq='daily')


def b_download_daily_option(data_type: str,
                            all_period: list = None,
                            start: datetime = None,
                            end: datetime = None,
                            all_symbols: list = None):
    """return date is: [start, end)"""

    global pre_url
    if all_period is None:
        all_period = ALL_PERIODS
    if start is None:
        start = SPOT_START_DATE
    if end is None:
        end = END_DATE

    if all_symbols is None:
        ok, all_symbols = get_all_option_symbols()
        if not ok:
            logger.warning(all_symbols)
            return

    b_download(data_type, all_period, start, end, all_symbols, spot='option', freq='daily')



from notecoin.huobi.history.const import *
from notecoin.huobi.history.download import *
from notecoin.huobi.history.utils import *

data_type = 'klines'


def download_daily_spot(all_period: list = None, start: datetime = None, end: datetime = None, all_symbols: list = None):
    '''return date is: [start, end)'''

    global data_type
    b_download_daily_spot(data_type, all_period, start, end, all_symbols)


def download_daily_future(
        all_period: list = None, start: datetime = None, end: datetime = None, all_symbols: list = None):
    '''return date is: [start, end)'''

    global data_type
    b_download_daily_future(data_type, all_period, start, end, all_symbols)


def download_daily_swap(all_period: list = None, start: datetime = None, end: datetime = None, all_symbols: list = None):
    '''return date is: [start, end)'''

    global data_type
    b_download_daily_swap(data_type, all_period, start, end, all_symbols)


def download_daily_linearswap(
        all_period: list = None, start: datetime = None, end: datetime = None, all_symbols: list = None):
    '''return date is: [start, end)'''

    global data_type
    b_download_daily_linearswap(data_type, all_period, start, end, all_symbols)


def download_daily_option(
        all_period: list = None, start: datetime = None, end: datetime = None, all_symbols: list = None):
    '''return date is: [start, end)'''

    global data_type
    b_download_daily_option(data_type, all_period, start, end, all_symbols)


if __name__ == "__main__":
    download_daily_spot(all_symbols=['BTCUSDT', 'ADAUSDT'],
                        start=datetime(2021, 5, 21),
                        end=datetime(2021, 5, 23),
                        all_period=['1min', '15min'])
    # download_daily_future()
    # download_daily_swap()
    # download_daily_linearswap()
    # download_daily_option(all_period=['1min'])



from notecoin.huobi.client import GenericClient, MarketClient
from notecoin.huobi.dataset.core import *
from notetool.tool.secret import read_secret
from tqdm import tqdm
import time

class TradeUpdate:
    def __init__(self, db_path='/root/workspace/tmp/coin/huobi.db'):
        api_key = read_secret(cate1='coin', cate2='huobi', cate3='api_key')
        secret_key = read_secret(cate1='coin', cate2='huobi', cate3='secret_key')
        self.market = MarketClient(api_key=api_key, secret_key=secret_key)
        self.generic = GenericClient(api_key=api_key, secret_key=secret_key)

        self.symbolDB = SymbolInfo(db_path=db_path)
        self.tradeDB = TradeDetail(db_path=db_path, conn=self.symbolDB.conn)
        self.kline_1min = Kline1MinDetail(db_path=db_path, conn=self.symbolDB.conn)
        self.kline_5min = Kline5MinDetail(db_path=db_path, conn=self.symbolDB.conn)
        self.kline_15min = Kline15MinDetail(db_path=db_path, conn=self.symbolDB.conn)
        self.kline_30min = Kline30MinDetail(db_path=db_path, conn=self.symbolDB.conn)
        self.kline_60min = Kline60MinDetail(db_path=db_path, conn=self.symbolDB.conn)
        self.kline_4hour = Kline4HourDetail(db_path=db_path, conn=self.symbolDB.conn)
        self.kline_1day = Kline1DayDetail(db_path=db_path, conn=self.symbolDB.conn)
        self.kline_1mon = Kline1MonDetail(db_path=db_path, conn=self.symbolDB.conn)
        self.kline_1week = Kline1WeekDetail(db_path=db_path, conn=self.symbolDB.conn)
        self.kline_1year = Kline1YearDetail(db_path=db_path, conn=self.symbolDB.conn)

        self.init()

    def init(self):
        self.symbolDB.create()
        self.tradeDB.create()
        self.kline_1min.create()
        self.kline_5min.create()
        self.kline_15min.create()
        self.kline_30min.create()
        self.kline_60min.create()
        self.kline_4hour.create()
        self.kline_1day.create()
        self.kline_1mon.create()
        self.kline_1week.create()
        self.kline_1year.create()

    @staticmethod
    def replace_key(data_dicts: list, symbol=None):
        res = []
        for _data_dict in data_dicts:
            data_dict = {"symbol": symbol}
            for key in _data_dict.keys():
                data_dict[key.replace('-', '_')] = _data_dict[key]
            res.append(data_dict)
        return res

    def insert_symbol(self):
        self.symbolDB.insert_list(self.replace_key(self.generic.get_exchange_symbols().data))

    def insert_trade(self, symbol):
        history = self.market.get_history_trade(symbol=symbol, size=2000)
        datas = history.dict_data['data']
        result = []
        [result.extend(data['data']) for data in datas]
        self.tradeDB.insert_list(self.replace_key(result, symbol=symbol))

    def insert_kline(self, symbol):
        self.kline_1min.insert_list(self.replace_key(self.market.get_candlestick(
            symbol=symbol, period='1min', size=2000).dict_data['data'], symbol))
        self.kline_5min.insert_list(self.replace_key(self.market.get_candlestick(
            symbol=symbol, period='5min', size=2000).dict_data['data'], symbol))
        self.kline_15min.insert_list(self.replace_key(self.market.get_candlestick(
            symbol=symbol, period='15min', size=2000).dict_data['data'], symbol))
        self.kline_30min.insert_list(self.replace_key(self.market.get_candlestick(
            symbol=symbol, period='30min', size=2000).dict_data['data'], symbol))
        self.kline_60min.insert_list(self.replace_key(self.market.get_candlestick(
            symbol=symbol, period='60min', size=2000).dict_data['data'], symbol))
        self.kline_4hour.insert_list(self.replace_key(self.market.get_candlestick(
            symbol=symbol, period='4hour', size=2000).dict_data['data'], symbol))
        self.kline_1day.insert_list(self.replace_key(self.market.get_candlestick(
            symbol=symbol, period='1day', size=2000).dict_data['data'], symbol))
        self.kline_1mon.insert_list(self.replace_key(self.market.get_candlestick(
            symbol=symbol, period='1mon', size=2000).dict_data['data'], symbol))
        self.kline_1week.insert_list(self.replace_key(self.market.get_candlestick(
            symbol=symbol, period='1week', size=2000).dict_data['data'], symbol))
        self.kline_1year.insert_list(self.replace_key(self.market.get_candlestick(
            symbol=symbol, period='1year', size=2000).dict_data['data'], symbol))

    def auto_save(self):
        self.symbolDB.auto_save()
        self.tradeDB.auto_save()
        self.kline_1min.auto_save()
        self.kline_5min.auto_save()
        self.kline_15min.auto_save()
        self.kline_30min.auto_save()
        self.kline_60min.auto_save()
        self.kline_4hour.auto_save()
        self.kline_1day.auto_save()
        self.kline_1mon.auto_save()
        self.kline_1week.auto_save()
        self.kline_1year.auto_save()

    def insert_data(self, trade=True, kline=True):
        symbols = self.symbolDB.select("select symbol from {} where state ='online'".format(self.symbolDB.table_name))
        for symbol in tqdm(symbols):
            try:
                symbol = symbol['symbol']
                if trade:
                    self.insert_trade(symbol)
                if kline:
                    self.insert_kline(symbol)
                time.sleep(0.3)
            except Exception as e:
                print(e)

        print(self.symbolDB.select('select count(1) as num from ' + self.symbolDB.table_name))
        print(self.tradeDB.select('select count(1) as num from ' + self.tradeDB.table_name))
        print("1min\t{}".format(self.kline_1min.select('select count(1) as num from ' + self.kline_1min.table_name)))
        print("5min\t{}".format(self.kline_5min.select('select count(1) as num from ' + self.kline_5min.table_name)))
        print("15min\t{}".format(self.kline_15min.select('select count(1) as num from ' + self.kline_15min.table_name)))
        print("30min\t{}".format(self.kline_30min.select('select count(1) as num from ' + self.kline_30min.table_name)))
        print("60min\t{}".format(self.kline_60min.select('select count(1) as num from ' + self.kline_60min.table_name)))
        print("4hour\t{}".format(self.kline_4hour.select('select count(1) as num from ' + self.kline_4hour.table_name)))
        print("1day\t{}".format(self.kline_1day.select('select count(1) as num from ' + self.kline_1day.table_name)))
        print("1week\t{}".format(self.kline_1week.select('select count(1) as num from ' + self.kline_1week.table_name)))
        print("1mon\t{}".format(self.kline_1mon.select('select count(1) as num from ' + self.kline_1mon.table_name)))
        print("1year\t{}".format(self.kline_1year.select('select count(1) as num from ' + self.kline_1year.table_name)))

    def run(self):
        t = 0

        while True:
            self.insert_symbol()
            if t % 100 == 0:
                self.insert_data(trade=False, kline=True)
                self.auto_save()
            else:
                self.insert_data(trade=False, kline=False)
                time.sleep(120)
            t += 1

    def test(self):
        import pandas as pd
        condition = 'id>=1619400078439 and id<=1619452799999 limit 111'
        sql = "select * from {} where {}".format(self.kline_1day.table_name, condition)
        print(sql)
        result = pd.read_sql(sql=sql, con=self.kline_1day.conn)
        print(result.head(10))


update = TradeUpdate()
update.run()
# update.test()

# ps -elf|grep update
# nohup /root/anaconda3/bin/python3.8 /root/workspace/notechats/notecoin/notecoin/huobi/dataset/update.py >>/root/workspace/tmp/notecoin-run-$(date +%Y-%m-%d).log 2>&1 &
# 2509
# sqlite3 -header -csv huobi.db "select * from trade_detail;" > trade_detail.csv



import json
import urllib.parse


class UrlParamsBuilder(object):

    def __init__(self):
        self.param_map = dict()
        self.post_map = dict()
        self.post_list = list()

    def put_url(self, name, value):
        if value is not None:
            if isinstance(value, (list, dict)):
                self.param_map[name] = value
            else:
                self.param_map[name] = str(value)

    def put_post(self, name, value):
        if value is not None:
            if isinstance(value, (list, dict)):
                self.post_map[name] = value
            else:
                self.post_map[name] = str(value)

    def build_url(self):
        if len(self.param_map) == 0:
            return ""
        encoded_param = urllib.parse.urlencode(self.param_map)
        return "?" + encoded_param

    def build_url_to_json(self):
        return json.dumps(self.param_map)

